package com.ringlord.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.Font;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingWorker;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Element;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.html.HTMLDocument;


/**
 * An "About" dialog, consisting of a logo image and an HTML display component
 * that supports hyperlinks whose activation launches the relevant URL using the
 * system web browser. The HTML component supports hover-highlighting of hyper
 * links (coloring is currently hard coded).
 * 
 * @author K Udo Schuermann
 */
public class About
  extends JPanel
{
  public About( final String htmlText,
	        final Font font,
	        final Icon logo )
  {
    super( new BorderLayout() );
    setBorder( BorderFactory.createEmptyBorder( 12,
	                                        12,
	                                        12,
	                                        12 ) );

    final JPanel iconPane = new JPanel( new BorderLayout() );
    iconPane.add( BorderLayout.NORTH,
	          new JLabel( logo ) );
    iconPane.setBorder( BorderFactory.createEmptyBorder( 0,
	                                                 0,
	                                                 0,
	                                                 12 ) );

    final JEditorPane output = new JEditorPane();
    output.setFont( font );
    output.setContentType( "text/html" );
    output.setEditable( false );
    output.setBorder( BorderFactory.createEmptyBorder( 8,
	                                               8,
	                                               8,
	                                               8 ) );
    final HTMLDocument doc = (HTMLDocument)output.getDocument();
    final StyleContext ss = doc.getStyleSheet();
    final Style highlight = ss.addStyle( "HighlightLink",
	                                 null );
    output.addHyperlinkListener( new HyperlinkListener()
    {
      @Override
      public void hyperlinkUpdate( final HyperlinkEvent e )
      {
	if( e.getEventType() == HyperlinkEvent.EventType.ENTERED )
	  {
	    // Highlight the link over which the mouse is hovering
	    final Element anchor = e.getSourceElement();
	    final int start = anchor.getStartOffset();
	    final int end = anchor.getEndOffset();

	    highlight.addAttribute( StyleConstants.Foreground,
		                    Color.RED );
	    doc.setCharacterAttributes( start,
		                        end - start,
		                        highlight,
		                        false );
	  }
	else if( e.getEventType() == HyperlinkEvent.EventType.EXITED )
	  {
	    // Unhighlight the previously highlighted link
	    final Element anchor = e.getSourceElement();
	    final int start = anchor.getStartOffset();
	    final int end = anchor.getEndOffset();

	    highlight.addAttribute( StyleConstants.Foreground,
		                    Color.BLUE );
	    doc.setCharacterAttributes( start,
		                        end - start,
		                        highlight,
		                        false );
	  }
	else if( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED )
	  {
	    // Launch the system web browser with the clicked link
	    final Desktop desktop = Desktop.getDesktop();
	    new SwingWorker<Void,Void>()
	    {
	      @Override
	      public Void doInBackground()
	      {
		try
		  {
		    desktop.browse( e.getURL().toURI() );
		  }
		catch( final URISyntaxException x )
		  {
		    x.printStackTrace( System.err );
		  }
		catch( final IOException x )
		  {
		    x.printStackTrace( System.err );
		  }
		catch( final Throwable x )
		  {
		    System.err.println( "Malformed URI? " + e.getURL() );
		    x.printStackTrace();
		  }
		return null;
	      }
	    }.execute();
	  }
      }
    } );

    final JScrollPane sOutput = new JScrollPane( output );
    // sOutput.setBorder( null );

    add( BorderLayout.WEST,
	 iconPane );
    add( BorderLayout.CENTER,
	 sOutput );

    output.setText( htmlText );
  }
  private static final long serialVersionUID = 7294034656545407785L;
}
