package com.ringlord.swing;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Path2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;
import javax.swing.border.AbstractBorder;


public class RoundedLineBorder
  extends AbstractBorder
{
  public static void main( final String[] args )
  {
    final JFrame f = new JFrame( "RoundedLineBorder Test" );
    f.setDefaultCloseOperation( WindowConstants.EXIT_ON_CLOSE );

    final JPanel q = new JPanel();
    q.setBackground( Color.green );
    q.setBorder( BorderFactory.createCompoundBorder( BorderFactory.createLineBorder( Color.red,
	                                                                             5 ),
	                                             new RoundedLineBorder( Color.blue,
	                                                                    5,
	                                                                    25,
	                                                                    Color.black ) ) );

    final JPanel p = new JPanel( new BorderLayout() );
    p.add( BorderLayout.NORTH,
	   new JLabel( "Top" ) );
    p.add( BorderLayout.SOUTH,
	   new JLabel( "Bottom" ) );
    p.add( BorderLayout.WEST,
	   new JLabel( "West" ) );
    p.add( BorderLayout.EAST,
	   new JLabel( "East" ) );
    p.add( BorderLayout.CENTER,
	   q );

    f.setContentPane( p );
    f.setSize( 320,
	       200 );
    f.setVisible( true );
  }


  public RoundedLineBorder( final Color lineColor,
	                    final int thickness,
	                    final int cornerRadius,
	                    final Color outsideColor )
  {
    super();
    this.lineColor = lineColor;
    this.thickness = thickness;
    this.cornerRadius = cornerRadius;
    this.outsideColor = outsideColor;
    this.stroke = new BasicStroke( thickness );
    this.halfThick = thickness / 2;
    this.insets = new Insets( cornerRadius / 2,
	                      cornerRadius / 2,
	                      cornerRadius / 2,
	                      cornerRadius / 2 );
  }


  public void paintBorder( final Component c,
	                   final Graphics g,
	                   int x,
	                   int y,
	                   int width,
	                   int height )
  {
    if( (this.thickness > 0) && (g instanceof Graphics2D) )
      {
	final Graphics2D g2 = (Graphics2D)g;
	final Color savedColor = g2.getColor();
	final Stroke savedStroke = g2.getStroke();

	g2.setColor( this.lineColor );
	g2.setStroke( this.stroke );

	g2.draw( new RoundRectangle2D.Double( x + halfThick,
	                                      y + halfThick,
	                                      width - thickness,
	                                      height - thickness,
	                                      cornerRadius,
	                                      cornerRadius ) );

	g2.setStroke( savedStroke );
	g2.setColor( savedColor );
      }
  }


  public Insets getBorderInsets( final Component c )
  {
    return new Insets( this.insets.top,
	               this.insets.left,
	               this.insets.bottom,
	               this.insets.right );
  }


  public Insets getBorderInsets( final Component c,
	                         final Insets insets )
  {
    if( insets == null )
      {
	return new Insets( this.insets.top,
	                   this.insets.left,
	                   this.insets.bottom,
	                   this.insets.right );
      }
    insets.set( this.insets.top,
	        this.insets.left,
	        this.insets.bottom,
	        this.insets.right );
    return insets;
  }
  private final Color lineColor, outsideColor;
  private final int thickness, cornerRadius, halfThick;
  private final Stroke stroke;
  private final Insets insets;
  //
  private static final long serialVersionUID = -5292059611781074742L;
}
